@@ -1,19 +1,17 @@
 package App::Mobirc;
-use Mouse;
-with 'App::Mobirc::Role::Plaggable';
+use Moose;
+with 'App::Mobirc::Role::Context', 'MooseX::Plaggerize';
 use 5.00800;
 use Scalar::Util qw/blessed/;
 use POE;
+use App::Mobirc::ConfigLoader;
 use App::Mobirc::Util;
 use UNIVERSAL::require;
 use Carp;
 use App::Mobirc::Model::Server;
 use Encode;
-use App::Mobirc::Types 'Config';
-use Text::MicroTemplate::File;
-use App::Mobirc::Web::Template;
 
-our $VERSION = '1.99_03';
+our $VERSION = '1.01';
 
 has server => (
     is      => 'ro',
@@ -24,36 +22,20 @@ has server => (
 
 has config => (
     is       => 'ro',
-    isa      => Config,
+    isa      => 'HashRef',
     required => 1,
-    coerce   => 1,
 );
 
-has mt => (
-    is => 'ro',
-    isa => 'Text::MicroTemplate::File',
-    lazy => 1,
-    default => sub {
-        my $self = shift;
-        Text::MicroTemplate::File->new(
-            include_path => [ File::Spec->catdir($self->config->{global}->{assets_dir}, 'tmpl') ],
-            package_name => "App::Mobirc::Web::Template",
-            use_cache    => 1,
-        );
-    },
-);
+around 'new' => sub {
+    my ($next, $class, $config_stuff) = @_;
+    my $config = App::Mobirc::ConfigLoader->load($config_stuff); # TODO: use coercing
 
-{
-    my $context;
-    sub context { $context }
-    sub _set_context { $context = $_[1] }
-}
+    my $self = $next->( $class, config => $config );
 
-sub BUILD {
-    my ($self, ) = @_;
     $self->_load_plugins();
-    $self->_set_context($self);
-}
+
+    return $self;
+};
 
 sub _load_plugins {
     my $self = shift;
@@ -75,7 +57,6 @@ sub run {
     $poe_kernel->run();
 }
 
-no Mouse; __PACKAGE__->meta->make_immutable;
 1;
 __END__
 
