@@ -1,24 +1,13 @@
 package App::Mobirc::Plugin::HTMLFilter::NickGroup;
 use strict;
-use App::Mobirc::Plugin;
+use MooseX::Plaggerize::Plugin;
 use List::Util qw/first/;
+use XML::LibXML;
 use Encode;
-use App::Mobirc::Validator;
-use HTML::TreeBuilder::XPath;
-use Mouse::Util::TypeConstraints;
-use JSON ();
-
-subtype 'HashRefEntry',
-    as 'HashRef';
-
-coerce 'HashRefEntry',
-    from 'Str',
-    via { JSON::from_json($_) };
 
 has 'map' => (
     is       => 'ro',
-    isa      => 'HashRefEntry',
-    coerce => 1,
+    isa      => 'HashRef',
     required => 1,
 );
 
@@ -41,26 +30,28 @@ has class_for => (
 );
 
 hook 'html_filter' => sub {
-    my ($self, $global_context, $req, $html) = validate_hook('html_filter', @_);
+    my ($self, $global_context, $c, $html) = @_;
 
-    my $tree = HTML::TreeBuilder::XPath->new;
-    $tree->parse_content($html);
+    my $doc = eval { XML::LibXML->new->parse_html_string($html); };
+    if ($@) {
+        warn $@;
+        return ($c, $html);
+    }
 
-    for my $elem ($tree->findnodes(q{//span[@class='nick_normal']})) {
+    for my $elem ($doc->findnodes(q{//span[@class='nick_normal']})) {
         if (my $who = $elem->findvalue('./text()')) {
             $who =~ s!^\((.+)\)$!$1!; # (who) => who
 
             if (my $new_class = $self->_class($who)) {
-                $elem->attr(class => $new_class);
+                $elem->setAttribute(class => $new_class);
             }
         }
     }
 
-    $html = $tree->as_HTML();
-
-    $tree = $tree->delete; # cleanup
-
-    return ($req, decode_utf8($html));
+    $html = $doc->toStringHTML();
+    $html =~ s{<!DOCTYPE[^>]*>\s*}{};
+    $html =~ s{(<a[^>]+)/>}{$1></a>}gi;
+    return ($c, decode($doc->encoding || "UTF-8", $html));
 };
 
 sub _class {
@@ -74,12 +65,6 @@ sub _class {
 }
 
 1;
-__END__
-
-=head1 SYNOPSIS
-
-    [HTMLFilter::NickGroup]
-    map={"subtech":['miyagawa','typester']}
 
 =head1 AUTHOR
 
